import json
import sys
from getpass import getpass

PWDB_PATH = 'pwdb.json'

def get_credentials():
    username = input('Enter your username: ')
    password = getpass('Enter your password: ')
    return (username, password)

def authenticate(username, password, pwdb):
    return pwhash(password) == pwdb[username]

def add_user(username, pwdb):
    password = getpass(f'Enter password for {username}: ')
    pwdb[username] = pwhash(password)
    return pwdb

def pwhash(password):
    ord_password_num = [int(ord(character)) for character in password]
    return sum(ord_password_num)

def read_pwdb(PWDB_PATH):
    try:
        pwdb_file = open(PWDB_PATH, 'rt')
        pwdb = json.load(pwdb_file)
    except Exception:
        pwdb = {}
    return pwdb

def write_pwdb(pwdb, PWDB_PATH):
    pwdb_file = open(PWDB_PATH, 'wt')
    json.dump(pwdb, pwdb_file)


if __name__ == "__main__":
    PWDB_PATH = 'pwdb.json'
    pwdb = read_pwdb(PWDB_PATH)

    if len(sys.argv) > 1:
        pwdb = add_user(sys.argv[1], pwdb)
        write_pwdb(pwdb, PWDB_PATH)
    else:
        username, password = get_credentials()
        if username not in pwdb:
            print('Wrong username!')
        else:
            if authenticate(username, password, pwdb):
                print('Successfully authenticated!')
            else:
                print('Wrong password!')

